from __future__ import absolute_import     
from __future__ import print_function

import numpy as np
from numba import int32, float64
import numba

class NumbaData(object):
    """A base class to extract data from derived class instances and use it as argument to @jit functions. Derived classes must set the 'signature' attribute."""
    def data(self):
        """Returns a jitclass instance containing attributes copied from self, using self.signature which is of the form type,names,type,names ... where names is a string 'attr1 attr2 attr3' containing space-separated names of attributes of self. Those attributes are declared to numba with the type preceding them. The result of data() can be used as argument to a @jit function."""
        spec = []
        for item in self.signature:
            if isinstance(item,str): 
                spec = spec + [(name,thetype) for name in item.split(' ')]
            else:
                thetype=item
        @numba.jitclass(spec)
        class JitClass(object):
            def __init__(self): pass
        data=JitClass()
        for name,thetype in spec: setattr(data, name, getattr(self,name))
        return data

jit=numba.jit(nopython=True, nogil=True, error_model='numpy', fastmath=True)
